<?php

function drupalcamp_solr($op = 'search', $keys = NULL, $filters = NULL, $solrsort = NULL, $path = NULL) {
   switch ($op) {
    case 'name':
      return t('Search');
    case 'reset':
      apachesolr_clear_last_index('apachesolr_search');
      return;
    case 'status':
      return apachesolr_index_status('apachesolr_search');

    case 'search':
    case 'search_indiv':
      //Modify filters acquisition in order to maintain this order: argument has preference from $_GET
      $filters_get = isset($_GET['filters']) ? $_GET['filters'] : '';
      if ( $filters ) {
        //check if our current filter is not included in the new filters
        if ( strstr( $filters, trim($filters_get) ) === false ) {
          $filters .= " $filters_get";
        }
      } else {
        $filters = $filters_get;
      }

      //Modify solrsort acquisition in order to maintain this order: argument has preference
      if( !$solrsort ) {
        $solrsort = isset($_GET['solrsort']) ? $_GET['solrsort'] : '';
      }

      if( !$page ) {
        $page = isset($_GET['page']) ? $_GET['page'] : 0;
      }

   try {
        if( $op == 'search' ) {
          $results = apachesolr_search_execute($keys, $filters, $solrsort, $path, $page);
        } else if( $op == 'search_indiv') {
          $results = drupalcamp_solr_execute($keys, $filters, $solrsort, $path.'/', $page);
        }
        return $results;
      }
      catch (Exception $e) {
        watchdog('Apache Solr', nl2br(check_plain($e->getMessage())), NULL, WATCHDOG_ERROR);
        apachesolr_failure(t('Solr search'), $keys);
      }
      break;
    } // switch
}


function drupalcamp_solr_view($type = NULL) {
  $keys = trim(search_get_keys());
  $content = '';
  // Search form submits with POST but redirects to GET.
  if (!isset($_POST['keys'])) {
    if (empty($type)) {
      // Note: search/X can not be a default tab because it would take on the
      // path of its parent (search). It would prevent remembering keywords when
      // switching tabs. This is why we drupal_goto to it from the parent instead.
      drupal_goto('search/apachesolr_search');
    }
    $filters = '';
    if (isset($_GET['filters'])) {
      $filters = trim($_GET['filters']);
    }

    // Only perform search if there is non-whitespace search term or filters:
    if ($keys || $filters || variable_get('apachesolr_search_browse', 'browse') == 'results') {
      if (variable_get('apachesolr_logging', TRUE)) {
        // Log the search keys:
        $log = $keys;
        if ($filters) {
          $log .= 'filters='. $filters;
        }
        watchdog('search', '%keys (@type).', array('%keys' => $log, '@type' => t('Search')), WATCHDOG_NOTICE, l(t('results'), 'search/'. $type .'/'. $keys));
      }

      // Collect the search results. See search_data().
      //$content = apachesolr_search_search('search', $keys);
      $content = drupalcamp_solr('search', $keys, $filters, null, 'search/apachesolr_search');
      if (isset($content) && is_array($content) && count($content)) {
        if (module_hook($type, 'search_page')) {
          $output = module_invoke($type, 'search_page', $content);
        }
        else {
          $output = theme('search_results', $content, $type);
        }
      }

      $filters = check_plain($filters);
      $apachesolr_has_searched = apachesolr_has_searched();
      if (variable_get('apachesolr_search_spellcheck', TRUE) && $apachesolr_has_searched && ($response = apachesolr_static_response_cache())) {
        // Get spellchecker suggestions into an array.
        if (isset($response->spellcheck->suggestions) && $response->spellcheck->suggestions) {
          $suggestions = get_object_vars($response->spellcheck->suggestions);
          if ($suggestions) {
            // Get the original query and replace words.
            $query = apachesolr_current_query();

            foreach ($suggestions as $word => $value) {
              $replacements[$word] = $value->suggestion[0];
            }
            $new_keywords = strtr($query->get_query_basic(), $replacements);

            // Show only if suggestion is different than current query.
            if ($query->get_query_basic() != $new_keywords) {
              $correction = t('Did you mean').' '.l($new_keywords, $query->get_path($new_keywords));
            }
          }
        }
      }
      $output_advice = theme('drupalcamp_solr_result_found', $content, $keys, $filters, $correction);
      return $output_advice.$output.theme('drupalcamp_solr_form_refine_search', drupal_get_form('search_form', NULL, $keys, $type));
    }
    elseif ($type != 'node') {
      // Ignore $type == node. Since we override the menu path to search to point
      // to this function, we have to count on core searches coming in here, too.
      switch (variable_get('apachesolr_search_browse', 'browse')) {
        case 'browse':
          try {
            // Show search form and browse-by blocks.
            $blocks = apachesolr_search_browse('', '', '', 'search/' . $type);
            if (count($blocks) > 0) {
              $content = theme('apachesolr_browse_blocks', $blocks);
            }
          }
          catch (Exception $e) {
            watchdog('Apache Solr', nl2br(check_plain($e->getMessage())), NULL, WATCHDOG_ERROR);
            apachesolr_failure(t('Solr search'), $keys);
          }
          break;
        case 'blocks':
          // Launch search so that hook_block() will show blocks.
          $dummy = search_data($keys, $type);
          break;
        case 'none':
          drupal_set_message(t('You cannot search on empty'), 'error', FALSE);
          break;
        default:
          break;
      }
    }
  }
  return $content.drupal_get_form('search_form', NULL, $keys, $type);
}

function drupalcamp_solr_custom($type = 'apachesolr_search', $path = 'search/drupalcamp', $filters = NULL, $sort = NULL) {
  $path_exploded = explode('/',$path);
  $key_level = count($path_exploded);
  $keys = trim(drupalcamp_get_keys($key_level));
  $content = '';
  if(empty($path)) {
    $path = 'search/apachesolr_search';
  }
  // Search form submits with POST but redirects to GET.
  if (!isset($_POST['keys'])) {
    if (empty($type)) {
      // Note: search/X can not be a default tab because it would take on the
      // path of its parent (search). It would prevent remembering keywords when
      // switching tabs. This is why we drupal_goto to it from the parent instead.
      drupal_goto('/'.$path);
    }
    //query is modified by query_alter to filter on specific content type
    if (isset($_GET['filters'])) {
      $filters .= trim($_GET['filters']);
    }

    if ($keys || $filters || variable_get('apachesolr_search_browse', 'browse') == 'results') {
      if (variable_get('apachesolr_logging', TRUE)) {
        // Log the search keys:
        $log = $keys;
        if ($filters) {
          $log .= 'filters='. $filters;
        }
        watchdog('search', '%keys (@type).', array('%keys' => $log, '@type' => t('Search')), WATCHDOG_NOTICE, l(t('results'), 'search/'. $type .'/'. $keys));
      }
      // Collect the search results. See search_data().

      $content = drupalcamp_solr('search', $keys, $filters, $sort, $path);
      $results = theme('search_results', $content, 'apachesolr_search');

      $output_advice = theme('drupalcamp_solr_result_found', $content, $keys, $filters);
      return $output_advice.$results;

      /*if ($results) {
        $output = theme('box', t('Search results'), $results);
      }
      else {
        $output = theme('box', t('Your search yielded no results'), theme('apachesolr_search_noresults'));
      }*/
    }
  }
  return $output;

}



function drupalcamp_get_keys($key_level) {
  static $return;
  if (!isset($return)) {
    $parts = explode('/', $_GET['q']);
    if (count($parts) == $key_level+1) {
      $return = array_pop($parts);
    }
    else {
      $return = empty($_REQUEST['keys']) ? '' : $_REQUEST['keys'];
    }
  }
  return $return;
}




/**
 * Execute a search results based on keyword, filter, and sort strings.
 *
 * @param $keys
 * @param $filterstring
 * @param $solrsort
 * @param $base_path
 *   For constructing filter and sort links. Leave empty unless the links need to point somewhere
 *   other than the base path of the current request.
 * @param integer $page
 *   For pagination.
 * @param $caller
 * @return Apache_Solr_Response $response
 * @throws Exception
 */
function drupalcamp_solr_execute($keys, $filterstring, $solrsort, $base_path = '', $page = 0, $caller = 'drupalcamp_solr') {
  $params = array();
  // This is the object that knows about the query coming from the user.
  $query = apachesolr_drupal_query($keys, $filterstring, $solrsort, $base_path);
  if (empty($query)) {
    throw new Exception(t('Could not construct a Solr query in function apachesolr_search_search()'));
  }

  $params += apachesolr_search_basic_params($query);
  if ($keys) {
    $params += apachesolr_search_highlighting_params($query);
    $params += apachesolr_search_spellcheck_params($query);
  }
  else {
    // No highlighting, use the teaser as a snippet.
    $params['fl'] .= ',teaser';
  }
  if (module_exists('upload')) {
    $params['fl'] .= ',is_upload_count';
  }
  apachesolr_search_add_facet_params($params, $query);
  apachesolr_search_add_boost_params($params, $query, apachesolr_get_solr());

  list($final_query, $response) = apachesolr_do_query($caller, $query, $params, $page);
  //Save numfound elements value
  variable_set('drupalcamp_solr_numfound', $response->response->numFound );
  return apachesolr_search_process_response($response, $final_query, $params);
}

function drupalcamp_solr_explode_filters($exploded_filters_array, $exclude_type = TRUE) {
  //added later in the process for higher reusability
  $output = NULL;
  if(is_array($exploded_filters_array)) {
    foreach($exploded_filters_array as $set_id => $exploded_set) {
      if($set_id == 'type' && $exclude_type){
        continue;
      }
      foreach ($exploded_set as $exploded_field) {
        $output .= $exploded_field.t(' - ');
      }
    }
    $output = substr($output,0,-3);
  }
  return $output;
}

function drupalcamp_solr_explode_filters_array($filters) {
  $exploded_filters = '';
  if(isset($filters) && is_string($filters)) {
  $filters = explode(' ', $filters);

    //put our filters in the search words
    foreach($filters as $filter) {
      $filter_parts = explode(':',$filter);
      $exploded_filters['cck'][] = $filter_parts[1];
    }
  }
  return $exploded_filters;
}




/**
 * Method to convert filter string to something human readable
 * @return string
 */
function drupalcamp_get_search_keys_to_string( $search_word = '', $filters = '' ) {
  if ( !apachesolr_has_searched() ) {
    //If we don't have searched anything and no search/filter has been passed, then return ''
    if( !$search_word && !$filters ) {
      return '';
    }
  }
  else if( $search_word || $filters ) {
    $filters = check_plain($filters);
    $exploded_filters_array = drupalcamp_solr_explode_filters_array($filters);
    $exploded_filters = drupalcamp_solr_explode_filters($exploded_filters_array, FALSE);
  }
  else {
    $filters = ( isset($_GET['filters']) ) ? trim($_GET['filters']) : '';
    $filters = check_plain($filters);
    $exploded_filters_array = drupalcamp_solr_explode_filters_array($filters);
    $exploded_filters = drupalcamp_solr_explode_filters($exploded_filters_array, FALSE);
    $search_word = arg(2);
  }
  if( arg(1) != 'apachesolr_search' ) {
    $search_word = '';
  }

  //Theme workarround.
  if( arg(0) == 'keys' ) {
    $search_word = arg(1);
  }

  if (empty($exploded_filters) && empty($search_word)) {
    $output_keys  = t("All");
  } else if(!empty($exploded_filters) && !empty($search_word)) {
    $output_keys = t('%search_string in %filters', array('%search_string' => $search_word, '%filters' => $exploded_filters));
  } else if(!empty($exploded_filters) && empty($search_word)) {
    $output_keys = t('All in %filters', array('%search_string' => $search_word, '%filters' => $exploded_filters));
  } else if(empty($exploded_filters) && !empty($search_word)) {
    $output_keys = t('%search_string', array('%search_string' => $search_word));
  }
  return $output_keys;
}
